@@ -1,6 +1,6 @@
 ## Domain Registry Interface, Handling of contact data for EURid
 ##
-## Copyright (c) 2005-2009,2012,2013 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+## Copyright (c) 2005 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -10,20 +10,21 @@
 ## (at your option) any later version.
 ##
 ## See the LICENSE file that comes with this distribution for more details.
+#
+# 
+#
 #########################################################################################
 
 package Net::DRI::Data::Contact::EURid;
 
 use strict;
-use warnings;
-
 use base qw/Net::DRI::Data::Contact/;
+__PACKAGE__->mk_accessors(qw(type vat lang));
 
 use Net::DRI::DRD::EURid;
 use Net::DRI::Exception;
-use Net::DRI::Util;
 
-__PACKAGE__->register_attributes(qw(type vat lang));
+our $VERSION=do { my @r=(q$Revision: 1.3 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 
 =pod
 
@@ -36,17 +37,23 @@ Net::DRI::Data::Contact::EURid - Handle EURid contact data for Net::DRI
 This subclass of Net::DRI::Data::Contact adds accessors and validation for
 EURid specific data.
 
-=head1 METHODS
-
 The following accessors/mutators can be called in chain, as they all return the object itself.
 
-=head2 type()
+=over
 
-type of contact : billing, tech, registrant or onsite (mandatory)
+=item *
 
-=head2 vat()
+C<type()> type of contact : billing, tech, registrant or onsite (mandatory)
 
-vat number of contact
+=item *
+
+C<vat()> vat number of contact
+
+=item *
+
+C<lang()> language of contact, must be in Europe (mandatory)
+
+=back
 
 =head1 SUPPORT
 
@@ -66,7 +73,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005-2009,2012,2013 Patrick Mevzek <netdri@dotandco.com>.
+Copyright (c) 2005 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -93,11 +100,6 @@ sub validate
   Net::DRI::Exception::usererr_insufficient_parameters('Invalid contact information: voice/type/lang mandatory') unless ($self->voice() && $self->type() && $self->lang());
  }
 
- ## Lower limits than in EPP (other checks already done in superclass)
- push @errs,'name' if ($self->name() && grep { length($_) > 50 }  ($self->name()));
- push @errs,'org'  if ($self->org()  && grep { length($_) > 100 } ($self->org()));
-
-
  push @errs,'type' if ($self->type() && $self->type()!~m/^(?:billing|tech|registrant|onsite)$/);
  push @errs,'vat'  if ($self->vat()  && !Net::DRI::Util::xml_is_token($self->vat(),1,20));
  push @errs,'lang' if ($self->lang() && !exists($Net::DRI::DRD::EURid::LANGA2_EU{lc($self->lang())}));
@@ -115,17 +117,5 @@ sub validate
  return 1; ## everything ok.
 }
 
-sub init
-{
- my ($self,$what,$ndr)=@_;
- if ($what eq 'create')
- {
-  my $a=$self->auth();
-  $self->auth({pw=>''}) unless ($a && (ref($a) eq 'HASH') && exists($a->{pw}));
-  $self->srid('ABCD') unless defined $self->srid(); ## we can not choose the ID
- }
- return;
-}
-
 ####################################################################################################
 1;
