@@ -1,7 +1,6 @@
 ## Domain Registry Interface, EURid EPP extensions
 ##
-## Copyright (c) 2005,2007-2012,2018 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
-##               2014 Michael Kefeder <michael.kefeder@world4you.com>.
+## Copyright (c) 2005 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -11,22 +10,27 @@
 ## (at your option) any later version.
 ##
 ## See the LICENSE file that comes with this distribution for more details.
-####################################################################################################
+#
+# 
+#
+#########################################################################################
 
 package Net::DRI::Protocol::EPP::Extensions::EURid;
 
 use strict;
-use warnings;
 
 use base qw/Net::DRI::Protocol::EPP/;
 
 use Net::DRI::Data::Contact::EURid;
+use Net::DRI::Protocol::EPP::Message;
+
+our $VERSION=do { my @r=(q$Revision: 1.2 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 
 =pod
 
 =head1 NAME
 
-Net::DRI::Protocol::EPP::Extensions::EURid - EURid (.EU) EPP extensions (release 5.6) for Net::DRI
+Net::DRI::Protocol::EPP::Extensions::EURid - EURid EPP extensions for Net::DRI
 
 =head1 DESCRIPTION
 
@@ -50,8 +54,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005,2007-2012i,2018 Patrick Mevzek <netdri@dotandco.com>.
-              2014 Michael Kefeder <michael.kefeder@world4you.com>.
+Copyright (c) 2005 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -64,28 +67,36 @@ See the LICENSE file that comes with this distribution for more details.
 =cut
 
 ####################################################################################################
-
-sub setup
+sub new
 {
- my ($self,$rp)=@_;
- my $version=$self->version();
+ my $h=shift;
+ my $c=ref($h) || $h;
 
-## NOT handled : dss, dynUpdate, euridcom
-# $self->ns({epp => 'http://www.eurid.eu/xml/epp/epp-1.0'});
-# $self->ns({ map { $_ => 'http://www.eurid.eu/xml/epp/'.$_.'-1.0' } qw/extendedInfo pendingTransaction/ });
- $self->ns({ map { $_ => 'http://www.eurid.eu/xml/epp/'.$_.'-1.1' } qw/nsgroup/ });
- $self->capabilities('contact_update','status',undef); ## No changes in status possible for .EU domains/contacts
- $self->capabilities('domain_update','status',undef);
- $self->capabilities('domain_update','nsgroup',[ 'add','del']);
- $self->factories('contact',sub { return Net::DRI::Data::Contact::EURid->new(); });
- $self->default_parameters({domain_create => { auth => { pw => '' } } });
- return;
+ my ($drd,$version,$extrah)=@_;
+ my %e=map { $_ => 1 } (defined($extrah)? (ref($extrah)? @$extrah : ($extrah)) : ());
+
+ $e{'Net::DRI::Protocol::EPP::Extensions::EURid::Domain'}=1;
+ $e{'Net::DRI::Protocol::EPP::Extensions::EURid::Contact'}=1;
+ $e{'Net::DRI::Protocol::EPP::Extensions::NSgroup'}=1;
+ ## Sunrise should be added when calling, as it is not mandatory
+
+ my $self=$c->SUPER::new($drd,$version,[keys(%e)]); ## we are now officially a Net::DRI::Protocol::EPP object
+
+ $self->{ns}->{_main}=['http://www.eurid.eu/xml/epp/epp-1.0','epp-1.0.xsd'];
+ foreach my $w ('domain','contact','eurid','nsgroup')
+ {
+  $self->{ns}->{$w}=['http://www.eurid.eu/xml/epp/'.$w.'-1.0',$w.'-1.0.xsd'];
+ }
+
+ my $rcapa=$self->capabilities();
+ delete($rcapa->{contact_update}->{status});
+ my $rfact=$self->factories();
+ $rfact->{contact}=sub { return Net::DRI::Data::Contact::EURid->new()->srid('ABCD') };
+ $rfact->{message}=sub { my $m=Net::DRI::Protocol::EPP::Message->new(@_); $m->ns($self->{ns}); $m->version($version); return $m;};
+
+ bless($self,$c); ## rebless
+ return $self;
 }
 
-## TODO Keygroup momentarily not used, in order to upgrade it to -1.1
-## TODO same for nsgroup, but -1.0 is still alowed
-## TODO EURid::Message removed for now
-sub default_extensions { return qw/EURid::Session EURid::Domain EURid::Contact EURid::Registrar EURid::Notifications EURid::IDN NSgroup SecDNS/; }
-
 ####################################################################################################
 1;
