@@ -1,8 +1,7 @@
 ## Domain Registry Interface, EURid Contact EPP extension commands
-## (based on EURid registration_guidelines_v1_0E-epp.pdf)
+## (based on EURid registration_guidelines_v1_0B-epp.pdf)
 ##
-## Copyright (c) 2005,2008,2012,2013,2018-2019 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
-##               2014 Michael Kefeder <michael.kefeder@world4you.com>. All rights reserved.
+## Copyright (c) 2005 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -12,14 +11,16 @@
 ## (at your option) any later version.
 ##
 ## See the LICENSE file that comes with this distribution for more details.
-####################################################################################################
+#
+# 
+#
+#########################################################################################
 
 package Net::DRI::Protocol::EPP::Extensions::EURid::Contact;
 
 use strict;
-use warnings;
 
-use Net::DRI::Util;
+our $VERSION=do { my @r=(q$Revision: 1.1 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 
 =pod
 
@@ -49,8 +50,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005,2008,2012,2013,2018-2019 Patrick Mevzek <netdri@dotandco.com>.
-              2014 Michael Kefeder <michael.kefeder@world4you.com>.
+Copyright (c) 2005 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -67,7 +67,7 @@ See the LICENSE file that comes with this distribution for more details.
 sub register_commands
 {
  my ($class,$version)=@_;
- my %tmp=(
+ my %tmp=( 
           create            => [ \&create, undef ],
           update            => [ \&update, undef ],
           info              => [ undef, \&info_parse ],
@@ -76,45 +76,46 @@ sub register_commands
  return { 'contact' => \%tmp };
 }
 
-sub setup
+####################################################################################################
+
+sub build_command_extension
 {
- my ($class,$po,$version)=@_;
- foreach my $ns (qw/contact-ext/)
- {
-  $po->ns({ $ns => 'http://www.eurid.eu/xml/epp/'.$ns.'-1.1' });
- }
- return;
+ my ($mes,$epp,$tag)=@_;
+
+ my @ns=@{$mes->ns->{eurid}};
+ return $mes->command_extension_register($tag,sprintf('xmlns:eurid="%s" xsi:schemaLocation="%s %s"',$ns[0],$ns[0],$ns[1]));
 }
 
-####################################################################################################
 
 sub create
 {
  my ($epp,$contact)=@_;
+ my $mes=$epp->message();
 
  ## validate() has been called, we are sure that type & lang exists
  my @n;
- push @n,['contact-ext:type',$contact->type()];
- push @n,['contact-ext:vat',$contact->vat()] if $contact->vat();
- push @n,['contact-ext:lang',$contact->lang()];
+ push @n,['eurid:type',$contact->type()];
+ push @n,['eurid:vat',$contact->vat()] if $contact->vat();
+ push @n,['eurid:lang',$contact->lang()];
 
- $epp->message()->command_extension('contact-ext', ['create', @n]);
- return;
+ my $eid=build_command_extension($mes,$epp,'eurid:ext');
+ $mes->command_extension($eid,['eurid:create',['eurid:contact',@n]]);
 }
 
 sub update
 {
  my ($epp,$domain,$todo)=@_;
+ my $mes=$epp->message();
 
  my $newc=$todo->set('info');
  return unless ($newc && (defined($newc->vat()) || defined($newc->lang())));
 
  my @n;
- push @n,['contact-ext:vat',$newc->vat()]   if defined($newc->vat());
- push @n,['contact-ext:lang',$newc->lang()] if defined($newc->lang());
+ push @n,['eurid:vat',$newc->vat()]   if defined($newc->vat());
+ push @n,['eurid:lang',$newc->lang()] if defined($newc->lang());
 
- $epp->message()->command_extension('contact-ext', ['update', ['chg', @n]]);
- return;
+ my $eid=build_command_extension($mes,$epp,'eurid:ext');
+ $mes->command_extension($eid,['eurid:update',['eurid:contact',['eurid:chg',@n]]]);
 }
 
 sub info_parse
@@ -123,19 +124,17 @@ sub info_parse
  my $mes=$po->message();
  return unless $mes->is_success();
 
- my $infdata=$mes->get_extension('contact-ext','infData');
+ my $infdata=$mes->get_content('infData',$mes->ns('eurid'),1);
  return unless $infdata;
 
  my $s=$rinfo->{contact}->{$oname}->{self};
- foreach my $el (Net::DRI::Util::xml_list_children($infdata))
- {
-  my ($name,$c)=@$el;
-  if ($name=~m/^(type|vat|lang)$/)
-  {
-   $s->$1($c->textContent());
-  }
- }
- return;
+
+ my $el=$infdata->getElementsByTagNameNS($mes->ns('eurid'),'type');
+ $s->type($el->get_node(1)->getFirstChild()->getData());
+ $el=$infdata->getElementsByTagNameNS($mes->ns('eurid'),'vat');
+ $s->vat($el->get_node(1)->getFirstChild()->getData()) if defined($el->get_node(1));
+ $el=$infdata->getElementsByTagNameNS($mes->ns('eurid'),'lang');
+ $s->lang($el->get_node(1)->getFirstChild()->getData());
 }
 
 ####################################################################################################
